"""MIT License

Copyright (c) 2024 gunyu1019

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import datetime

from typing import Annotated, List, Optional, Generic, TypeVar
from pydantic import BeforeValidator, Field

from ..base_model import ChzzkModel, ManagerClientAccessable
from ..user import PartialUser
from ..video import PartialVideo

T = TypeVar("T")


class ManageResult(ChzzkModel, Generic[T]):
    page: int
    size: int
    total_count: int
    total_pages: int
    data: List[T]

    def __iter__(self):
        return self.data.__iter__()

    def __len__(self):
        return self.total_count

    def __getitem__(self, index: int):
        return self.data[index]


class FollowingInfo(ChzzkModel):
    following: bool
    notification: bool
    follow_date: Optional[datetime.datetime]


class ManageSubcriber(ChzzkModel):  # incomplete data
    user_id_hash: Optional[str]
    nickname: Optional[str]
    profile_image_url: Optional[str]
    verified_mark: bool = False

    total_month: int
    tier: str
    publish_period: int
    twitch_month: Optional[int] = None
    created_date: datetime.datetime


class ManageFollower(ChzzkModel):
    user: PartialUser
    following: FollowingInfo
    channel_following: FollowingInfo


class RestrictUser(ChzzkModel, ManagerClientAccessable):
    seq: int
    user_id_hash: str
    nickname: str
    execute_nickname: str
    created_date: datetime.datetime

    @ManagerClientAccessable.based_manage_client
    async def remove_restrict(self):
        """Remove this user to restrict activity."""
        await self._manage_client.remove_restrict(self.user_id_hash)


class UnrestrictRequest(ChzzkModel, ManagerClientAccessable):
    request_no: int
    restrict_seq: int
    user: PartialUser = Field(alias="userResponse")
    vindication: str
    created_date: datetime.datetime

    @ManagerClientAccessable.based_manage_client
    async def approve(self):
        """Approve this unrestrict activity request."""
        await self._manage_client.remove_restrict(self.user_id_hash)

    @ManagerClientAccessable.based_manage_client
    async def reject(self, reason: str):
        """Deny this unrestrict activity request."""
        await self._manage_client._http.reject_unrestrict_request(
            channel_id=self.channel_id, request_number=self.request_no, judgment=reason
        )


class ManageVideo(PartialVideo):
    live_id: Optional[int] = None
    created_date: Annotated[
        datetime.datetime, BeforeValidator(ChzzkModel.special_date_parsing_validator)
    ]
    like_count: int
    read_count: int
    comment_count: int
    vod_status: str
    exposure: bool
    publish: bool
    publish_type: str
    manual_publishable: bool
    exposure_button: str
    live_accumulate_count: int = 0
    live_unique_view_count: int = 0
    live_open_date: Annotated[
        Optional[datetime.datetime],
        BeforeValidator(ChzzkModel.special_date_parsing_validator),
    ] = None
    deleted: bool
    # download
    # deletedBy: Type Unknown??? // Nullable



"""MIT License

Copyright (c) 2024 gunyu1019

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import aiohttp
import asyncio
import datetime
import logging
from typing import Any, Optional, Callable, Coroutine, Literal, TYPE_CHECKING

from .enums import ChatCmd
from .error import ChatConnectFailed
from .gateway import ChzzkWebSocket, ReconnectWebsocket
from .http import ChzzkAPIChatSession, NaverGameChatSession
from .state import ConnectionState
from ..client import Client
from ..error import LoginRequired
from ..manage.manage_client import ManageClient
from ..user import PartialUser
from ..live import LiveDetail, LiveStatus
from ..http import ChzzkAPISession

if TYPE_CHECKING:
    from .access_token import AccessToken
    from .message import ChatMessage
    from .profile import Profile
    from .recent_chat import RecentChat

_log = logging.getLogger(__name__)


class ChatClient(Client):
    """Represents a client to connect Chzzk (Naver Live Streaming).
    Addition, this class includes chat feature.
    """

    def __init__(
        self,
        channel_id: str,
        authorization_key: Optional[str] = None,
        session_key: Optional[str] = None,
        chat_channel_id: Optional[str] = None,
        loop: Optional[asyncio.AbstractEventLoop] = None,
    ):
        super().__init__(
            loop=loop, authorization_key=authorization_key, session_key=session_key
        )

        self.chat_channel_id: str = chat_channel_id
        self.channel_id: str = channel_id
        self.access_token: Optional[AccessToken] = None
        self.user_id: Optional[str] = None

        self.ws_session = aiohttp.ClientSession(loop=self.loop)

        self._listeners: dict[str, list[tuple[asyncio.Future, Callable[..., bool]]]] = (
            dict()
        )
        self._extra_event: dict[str, list[Callable[..., Coroutine[Any, Any, Any]]]] = (
            dict()
        )

        self._ready = asyncio.Event()

        handler = {ChatCmd.CONNECTED: self._ready.set}
        self._connection = ConnectionState(
            dispatch=self.dispatch, handler=handler, client=self
        )
        self._gateway: Optional[ChzzkWebSocket] = None
        self._status: Literal["OPEN", "CLOSE"] = None

    def _session_initial_set(self):
        self._api_session = ChzzkAPIChatSession(loop=self.loop)
        self._game_session = NaverGameChatSession(loop=self.loop)

    @property
    def is_connected(self) -> bool:
        """Specifies if the client successfully connected with chzzk."""
        return self._ready.is_set()

    def run(self, authorization_key: str = None, session_key: str = None) -> None:
        wrapper = self.start(authorization_key, session_key)
        try:
            self.loop.run_until_complete(wrapper)
        except KeyboardInterrupt:
            return

    async def start(self, authorization_key: str = None, session_key: str = None):
        try:
            if authorization_key is not None and session_key is not None:
                self.login(authorization_key=authorization_key, session_key=session_key)
            await self.connect()
        finally:
            await self.close()

    def login(self, authorization_key: str, session_key: str):
        """Login at Chzzk.
        Used for features that require a login. (ex. user method)

        Parameters
        ----------
        authorization_key : str
            A `NID_AUT` value in the cookie.
        session_key : str
            A `NID_SES` value in the cookie.
        """
        super().login(authorization_key=authorization_key, session_key=session_key)
        self._manage_client[self.channel_id] = ManageClient(self.channel_id, self)

    async def connect(self) -> None:
        if self.chat_channel_id is None:
            status = await self.live_status(channel_id=self.channel_id)
            if status is None:
                raise ChatConnectFailed.channel_is_null(self.channel_id)

            if (
                status.adult
                and status.user_adult_status != "ADULT"
                and status.chat_channel_id is None
            ):
                raise ChatConnectFailed.adult_channel(self.channel_id)
            elif status.chat_channel_id is None:
                raise ChatConnectFailed.chat_channel_is_null()

            self.chat_channel_id = status.chat_channel_id
            self._status = status.status

        if self._game_session.has_login:
            user = await self.user()
            self.user_id = user.user_id_hash

        if self.access_token is None:
            await self._generate_access_token()

        await self.polling()

    async def close(self):
        """Close the connection to chzzk."""
        self._ready.clear()

        if self._gateway is not None:
            await self._gateway.close()
        await self.ws_session.close()
        await super().close()

    async def _confirm_live_status(self):
        live_status = await self.live_status(channel_id=self.channel_id)
        if live_status is None:
            return

        if self._status != live_status.status:
            self._status = live_status.status
            if self._status == "OPEN":
                self.dispatch("broadcast_open")
            elif self._status == "CLOSE":
                self.dispatch("broadcast_close")

        if live_status.chat_channel_id == self.chat_channel_id:
            return

        _log.debug("A chat_channel_id has been updated. Reconnect websocket.")
        await self._gateway.close()

        self.chat_channel_id = live_status.chat_channel_id
        raise ReconnectWebsocket()

    async def polling(self) -> None:
        session_id: Optional[str] = None
        while not self.is_closed:
            try:
                self._gateway = await ChzzkWebSocket.from_client(
                    self, self._connection, session_id=session_id
                )

                # Initial Connection
                if session_id is None:
                    await self._gateway.send_open(
                        access_token=self.access_token.access_token,
                        chat_channel_id=self.chat_channel_id,
                        mode="READ" if self.user_id is None else "SEND",
                        user_id=self.user_id,
                    )
                    session_id = self._gateway.session_id

                last_check_time = datetime.datetime.now()

                while True:
                    await self._gateway.poll_event()

                    # Confirm chat-channel-id with live_status() method.
                    # When a streamer starts a new broadcast, a chat-channel-id will regenrated.
                    #
                    # https://github.com/gunyu1019/chzzkpy/issues/31
                    relative_time = datetime.datetime.now() - last_check_time
                    if relative_time.total_seconds() >= 58:
                        last_check_time = datetime.datetime.now()
                        await self._confirm_live_status()
            except ReconnectWebsocket:
                self.dispatch("disconnect")
                session_id = None
                continue

    # Event Handler
    async def wait_until_connected(self) -> None:
        """Waits until the client's internal cache is all ready."""
        await self._ready.wait()

    def wait_for(
        self,
        event: str,
        check: Optional[Callable[..., bool]] = None,
        timeout: Optional[float] = None,
    ):
        """Waits for a WebSocket event to be dispatched.

        Parameters
        ----------
        event : str
            The event name.
            For a list of events, read :meth:`event`
        check : Optional[Callable[..., bool]],
            A predicate to check what to wait for. The arguments must meet the
            parameters of the event being waited for.
        timeout : Optional[float]
            The number of seconds to wait before timing out and raising
            :exc:`asyncio.TimeoutError`.

        """
        future = self.loop.create_future()

        if check is None:

            def _check(*_):
                return True

            check = _check
        event_name = event.lower()

        if event_name not in self._listeners.keys():
            self._listeners[event_name] = list()
        self._listeners[event_name].append((future, check))
        return asyncio.wait_for(future, timeout=timeout)

    def event(
        self, coro: Callable[..., Coroutine[Any, Any, Any]]
    ) -> Callable[..., Coroutine[Any, Any, Any]]:
        """A decorator that registers an event to listen to.
        The function must be corutine. Else client cause TypeError


        A list of events that the client can listen to.
        * `on_chat`: Called when a ChatMessage is created and sent.
        * `on_connect`: Called when the client is done preparing the data received from Chzzk.
        * `on_donation`: Called when a listener donates
        * `on_system_message`: Called when a system message is created and sent.
                            (Example. notice/blind message)
        * `on_recent_chat`: Called when a recent chat received.
                            This event called when `request_recent_chat` method called.
        * `on_pin` / `on_unpin`: Called when a message pinned or unpinned.
        * `on_blind`: Called when a message blocked.
        * `on_client_error`: Called when client cause exception.

        Example
        -------
        >>> @client.event
        ... async def on_chat(message: ChatMessage):
        ...     print(message.content)
        """
        if not asyncio.iscoroutinefunction(coro):
            raise TypeError("function must be a coroutine.")

        event_name = coro.__name__
        if event_name not in self._listeners.keys():
            self._extra_event[event_name] = list()
        self._extra_event[event_name].append(coro)
        return coro

    def dispatch(self, event: str, *args: Any, **kwargs) -> None:
        _log.debug("Dispatching event %s", event)
        method = "on_" + event

        # wait-for listeners
        if event in self._listeners.keys():
            listeners = self._listeners[event]
            _new_listeners = []

            for index, (future, condition) in enumerate(listeners):
                if future.cancelled():
                    continue

                try:
                    result = condition(*args, **kwargs)
                except Exception as e:
                    future.set_exception(e)
                    continue
                if result:
                    match len(args):
                        case 0:
                            future.set_result(None)
                        case 1:
                            future.set_result(args[0])
                        case _:
                            future.set_result(args)

                _new_listeners.append((future, condition))
            self._listeners[event] = _new_listeners

        # event-listener
        if method not in self._extra_event.keys():
            return

        for coroutine_function in self._extra_event[method]:
            self._schedule_event(coroutine_function, method, *args, **kwargs)

    async def _run_event(
        self,
        coro: Callable[..., Coroutine[Any, Any, Any]],
        event_name: str,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        try:
            await coro(*args, **kwargs)
        except asyncio.CancelledError:
            pass
        except Exception as exc:
            try:
                _log.exception("Ignoring exception in %s", event_name)
                self.dispatch("error", exc, *args, **kwargs)
            except asyncio.CancelledError:
                pass

    def _schedule_event(
        self,
        coro: Callable[..., Coroutine[Any, Any, Any]],
        event_name: str,
        *args: Any,
        **kwargs: Any,
    ) -> asyncio.Task:
        wrapped = self._run_event(coro, event_name, *args, **kwargs)
        # Schedules the task
        return self.loop.create_task(wrapped, name=f"chzzk.py: {event_name}")

    # API Method
    async def _generate_access_token(self) -> AccessToken:
        res = await self._game_session.chat_access_token(
            channel_id=self.chat_channel_id
        )
        self.access_token = res.content
        return self.access_token

    # Chat Method
    async def send_chat(self, message: str) -> None:
        """Send a message.

        Parameters
        ----------
        message : str
            Message to Broadcasters

        Raises
        ------
        RuntimeError
            Occurs when the client can't connect to a broadcaster's chat
        """
        if not self.is_connected:
            raise RuntimeError("Not connected to server. Please connect first.")

        if not self.user_id:
            raise LoginRequired()

        await self._gateway.send_chat(message, self.chat_channel_id)

    async def request_recent_chat(self, count: int = 50):
        """Send a request recent chat to chzzk.
        This method only makes a “request”.
        If you want to get the recent chats of participants, use the `history` method.

        Parameters
        ----------
        count : Optional[int]
            Number of messages to fetch from the most recent, by default 50

        Raises
        ------
        RuntimeError
            Occurs when the client can't connect to a broadcaster's chat
        """
        if not self.is_connected:
            raise RuntimeError("Not connected to server. Please connect first.")

        await self._gateway.request_recent_chat(count, self.chat_channel_id)

    async def history(self, count: int = 50) -> list[ChatMessage]:
        """Get messages the user has previously sent.

        Parameters
        ----------
        count : Optional[int]
            Number of messages to fetch from the most recent, by default 50

        Returns
        -------
        list[ChatMessage]
            Returns the user's most recently sent messages, in order of appearance
        """
        await self.request_recent_chat(count)
        recent_chat: RecentChat = await self.wait_for(
            "recent_chat", lambda x: len(x.message_list) <= count
        )
        return recent_chat.message_list

    async def set_notice_message(self, message: ChatMessage) -> None:
        """Set a pinned messsage.

        Parameters
        ----------
        message : ChatMessage
            A Chat to pin.
        """
        await self._game_session.set_notice_message(
            channel_id=self.chat_channel_id,
            extras=(
                message.extras.model_dump_json(by_alias=True)
                if message.extras is not None
                else "{}"
            ),
            message=message.content,
            message_time=int(message.created_time.timestamp() * 1000),
            message_user_id_hash=message.user_id,
            streaming_channel_id=message.extras.streaming_channel_id,
        )
        return

    async def delete_notice_message(self) -> None:
        """Delete a pinned message."""
        await self._game_session.delete_notice_message(channel_id=self.chat_channel_id)
        return

    async def blind_message(self, message: ChatMessage) -> None:
        """Blinds a chat.

        Parameters
        ----------
        message : ChatMessage
            A Chat to blind.
        """
        await self._game_session.blind_message(
            channel_id=self.chat_channel_id,
            message=message.content,
            message_time=int(message.created_time.timestamp() * 1000),
            message_user_id_hash=message.user_id,
            streaming_channel_id=message.extras.streaming_channel_id,
        )
        return

    async def temporary_restrict(self, user: PartialUser | str) -> PartialUser:
        """Give temporary restrict to user.
        A temporary restriction cannot be lifted arbitrarily.

        Parameters
        ----------
        user : ParticleUser | str
            A user object to give temporary restrict activity.
            Instead, it can be user id.

        Returns
        -------
        ParticleUser
            Returns an user temporary restricted in chat.
        """
        user_id = user
        if isinstance(user, PartialUser):
            user_id = user.user_id_hash

        response = await self._api_session.temporary_restrict(
            channel_id=self.channel_id,
            chat_channel_id=self.chat_channel_id,
            target_id=user_id,
        )
        return response

    async def live_status(self, channel_id: Optional[str] = None):
        """Get a live status info of broadcaster.

        Parameters
        ----------
        channel_id : Optional[str]
            The channel ID of broadcaster, default by channel id of ChatClient.

        Returns
        -------
        Optional[LiveStatus]
            Return LiveStatus info. Sometimes the broadcaster is not broadcasting, returns None.
        """
        if channel_id is None:
            channel_id = self.channel_id
        return await super().live_status(channel_id)

    async def live_detail(self, channel_id: Optional[str] = None):
        """Get a live detail info of broadcaster.

        Parameters
        ----------
        channel_id : Optional[str]
            The channel ID of broadcaster, default by channel id of ChatClient.

        Returns
        -------
        Optional[LiveDetail]
            Return LiveDetail info. Sometimes the broadcaster is not broadcasting, returns None.
        """
        if channel_id is None:
            channel_id = self.channel_id
        return await super().live_detail(channel_id)

    def manage(self, channel_id: Optional[str] = None) -> ManageClient:
        """Get a client provided broadcast management functionality.

        Parameters
        ----------
        channel_id : Optional[str]
            A channel id to manage broadcasts.
            The default value is the last channel id used.
            If initally use the manage method and don't have a channel_id argument,
            the default value is channel id of ChatClient.

        Returns
        -------
        ManageClient
            Return a client provided broadcast management functionality.
        """
        if channel_id is None and self._latest_manage_client_id is None:
            channel_id = self.channel_id
        return super().manage(channel_id=channel_id)

    @property
    def manage_self(self) -> ManageClient:
        """Get a client provided self-channel management functionally."""
        return self.manage(channel_id=self.channel_id)

    async def profile_card(self, user: PartialUser | str) -> Profile:
        """Get a profile card.

        Parameters
        ----------
        user : ParticleUser | str
            A user object to get profile card.
            Instead, it can be user id.

        Returns
        -------
        Profile
            Returns a profile card with this channel information (include following info).
        """
        user_id = user
        if isinstance(user, PartialUser):
            user_id = user.user_id_hash

        data = await self._game_session.profile_card(
            chat_channel_id=self.chat_channel_id, user_id=user_id
        )
        data.content._set_manage_client(self.manage_self)
        return data.content



"""MIT License

Copyright (c) 2024 gunyu1019

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import asyncio

from ahttp_client import get, post, put, delete, Path, Query
from ahttp_client.extension import get_pydantic_response_model
from typing import Annotated, Optional, Literal

from ..base_model import Content
from ..http import ChzzkSession
from ..user import PartialUser
from .chat_activity_count import ChatActivityCount
from .chat_rule import ChatRule
from .manage_search import (
    ManageResult,
    ManageFollower,
    ManageSubcriber,
    RestrictUser,
    UnrestrictRequest,
    ManageVideo,
)
from .prohibit_word import ProhibitWordResponse
from .stream import Stream


class ChzzkManageSession(ChzzkSession):
    def __init__(self, loop: Optional[asyncio.AbstractEventLoop] = None):
        super().__init__(base_url="https://api.chzzk.naver.com", loop=loop)

        self.add_restrict.before_hook(self.query_to_json)
        self.add_role.before_hook(self.query_to_json)
        self.add_prohibit_word.before_hook(self.query_to_json)
        self.edit_prohibit_word.before_hook(self.query_to_json)
        self.set_chat_rule.before_hook(self.query_to_json)
        self.reject_unrestrict_request.before_hook(self.query_to_json)

    @get_pydantic_response_model()
    @post("/manage/v1/channels/{channel_id}/restrict-users", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def add_restrict(
        self,
        channel_id: Annotated[str, Path],
        target_id: Annotated[str, Query.to_camel()],
    ) -> Content[PartialUser]:
        pass

    @get_pydantic_response_model()
    @delete(
        "/manage/v1/channels/{channel_id}/restrict-users/{target_id}",
        directly_response=True,
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def remove_restrict(
        self, channel_id: Annotated[str, Path], target_id: Annotated[str, Path]
    ) -> Content[None]:
        pass

    @get_pydantic_response_model()
    @post("/manage/v1/channels/{channel_id}/streaming-roles", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def add_role(
        self,
        channel_id: Annotated[str, Path],
        target_id: Annotated[str, Query.to_camel()],
        user_role_type: Annotated[
            Literal[
                "STREAMING_CHAT_MANAGER",
                "STREAMING_CHANNEL_MANAGER",
                "STREAMING_STATTLE_MANAGER",
            ],
            Query.to_camel(),
        ],
    ) -> Content[PartialUser]:
        pass

    @get_pydantic_response_model()
    @delete(
        "/manage/v1/channels/{channel_id}/streaming-roles/{target_id}",
        directly_response=True,
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def remove_role(
        self,
        channel_id: Annotated[str, Path],
        target_id: Annotated[str, Path],
    ) -> Content[None]:
        pass

    @get_pydantic_response_model()
    @get(
        "/manage/v1/channels/{channel_id}/chats/prohibit-words", directly_response=True
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def get_prohibit_words(
        self,
        channel_id: Annotated[str, Path],
    ) -> Content[ProhibitWordResponse]:
        pass

    @get_pydantic_response_model()
    @post(
        "/manage/v1/channels/{channel_id}/chats/prohibit-words", directly_response=True
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def add_prohibit_word(
        self,
        channel_id: Annotated[str, Path],
        prohibit_word: Annotated[str, Query.to_camel()],
    ) -> Content[None]:
        pass

    @get_pydantic_response_model()
    @delete(
        "/manage/v1/channels/{channel_id}/chats/prohibit-words/{prohibit_word_number}",
        directly_response=True,
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def remove_prohibit_word(
        self,
        channel_id: Annotated[str, Path],
        prohibit_word_number: Annotated[str, Path],
    ) -> Content[None]:
        pass

    @get_pydantic_response_model()
    @delete(
        "/manage/v1/channels/{channel_id}/chats/prohibit-words", directly_response=True
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def remove_prohibit_word_all(
        self,
        channel_id: Annotated[str, Path],
    ) -> Content[None]:
        pass

    @get_pydantic_response_model()
    @put(
        "/manage/v1/channels/{channel_id}/chats/prohibit-words/{prohibit_word_number}",
        directly_response=True,
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def edit_prohibit_word(
        self,
        channel_id: Annotated[str, Path],
        prohibit_word_number: Annotated[str, Path],
        prohibit_word: Annotated[str, Query.to_camel()],
    ) -> Content[None]:
        pass

    @get_pydantic_response_model()
    @get("/manage/v1/channels/{channel_id}/streams", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def stream(
        self,
        channel_id: Annotated[str, Path],
    ) -> Content[Stream]:
        pass

    @get_pydantic_response_model()
    @get("/manage/v1/channels/{channel_id}/chat-rules", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def get_chat_rule(
        self,
        channel_id: Annotated[str, Path],
    ) -> Content[ChatRule]:
        pass

    @get_pydantic_response_model()
    @put("/manage/v1/channels/{channel_id}/chat-rules", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def set_chat_rule(
        self, channel_id: Annotated[str, Path], rule: Annotated[str, Query.to_camel()]
    ) -> Content[None]:
        pass

    @get_pydantic_response_model()
    @get(
        "/manage/v1/channels/{channel_id}/users/{target_id}/chat-activity-count",
        directly_response=True,
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def chat_activity_count(
        self,
        channel_id: Annotated[str, Path],
        target_id: Annotated[str, Path],
    ) -> Content[ChatActivityCount]:
        pass

    @get_pydantic_response_model()
    @get("/manage/v1/channels/{channel_id}/subscribers", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def subcribers(
        self,
        channel_id: Annotated[str, Path],
        page: Annotated[int, Query.to_camel()] = 0,
        size: Annotated[int, Query.to_camel()] = 50,
        sort_type: Annotated[
            Optional[Literal["RECENT", "LONGER"]], Query.to_camel()
        ] = "RECENT",
        publish_period: Annotated[Optional[Literal[1, 3, 6]], Query.to_camel()] = "",
        tier: Annotated[Optional[Literal["TIER_1", "TIER_2"]], Query.to_camel()] = "",
        user_nickname: Annotated[Optional[str], Query.to_camel()] = "",
    ) -> Content[ManageResult[ManageSubcriber]]:
        pass

    @get_pydantic_response_model()
    @get("/manage/v1/channels/{channel_id}/followers", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def followers(
        self,
        channel_id: Annotated[str, Path],
        page: Annotated[int, Query.to_camel()] = 0,
        size: Annotated[int, Query.to_camel()] = 50,
        sort_type: Annotated[
            Optional[Literal["RECENT", "LONGER"]], Query.to_camel()
        ] = "RECENT",
    ) -> Content[ManageResult[ManageFollower]]:
        pass

    @get_pydantic_response_model()
    @get("/manage/v1/channels/{channel_id}/restrict-users", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def restricts(
        self,
        channel_id: Annotated[str, Path],
        page: Annotated[int, Query.to_camel()] = 0,
        size: Annotated[int, Query.to_camel()] = 50,
        user_nickname: Annotated[Optional[str], Query.to_camel()] = "",
    ) -> Content[ManageResult[RestrictUser]]:
        pass

    @get_pydantic_response_model()
    @get(
        "/manage/v1/channels/{channel_id}/restrict-release-requests",
        directly_response=True,
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def unrestrict_requests(
        self,
        channel_id: Annotated[str, Path],
        page: Annotated[int, Query.to_camel()] = 0,
        size: Annotated[int, Query.to_camel()] = 50,
        user_nickname: Annotated[Optional[str], Query.to_camel()] = "",
    ) -> Content[ManageResult[UnrestrictRequest]]:
        pass

    @get_pydantic_response_model()
    @get("/manage/v1/channels/{channel_id}/videos", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def videos(
        self,
        channel_id: Annotated[str, Path],
        video_type: Annotated[Literal["UPLOAD", "REPLAY"], Query.to_camel()],
        page: Annotated[int, Query.to_camel()] = 0,
        size: Annotated[int, Query.to_camel()] = 50,
    ) -> Content[ManageResult[ManageVideo]]:
        pass

    @get_pydantic_response_model()
    @put(
        "/manage/v1/channels/{channel_id}/restrict-release-requests/{request_number}/reject",
        directly_response=True,
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def reject_unrestrict_request(
        self,
        channel_id: Annotated[str, Path],
        request_number: Annotated[int, Path],
        judgment: Annotated[str, Query.to_camel()],
    ) -> Content[UnrestrictRequest]:
        pass



"""MIT License

Copyright (c) 2024 gunyu1019

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import asyncio

from typing import Optional, TYPE_CHECKING
from .http import ChzzkAPISession, NaverGameAPISession
from .live import Live, LiveStatus, LiveDetail
from .user import User
from .video import Video
from .manage.manage_client import ManageClient


if TYPE_CHECKING:
    from .channel import Channel
    from types import TracebackType
    from typing_extensions import Self


class Client:
    """Represents a client to connect Chzzk (Naver Live Streaming)."""

    def __init__(
        self,
        loop: Optional[asyncio.AbstractEventLoop] = None,
        authorization_key: Optional[str] = None,
        session_key: Optional[str] = None,
    ):
        self.loop = loop or asyncio.get_event_loop()
        self._closed = False
        self._api_session = None
        self._game_session = None

        if authorization_key is not None and session_key is not None:
            self.login(authorization_key, session_key)

        self._session_initial_set()

        # For management feature
        self._manage_client: dict[str, ManageClient] = dict()
        self._latest_manage_client_id = None

    def _session_initial_set(self):
        self._api_session = ChzzkAPISession(loop=self.loop)
        self._game_session = NaverGameAPISession(loop=self.loop)

    async def __aenter__(self) -> Self:
        return self

    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> None:
        if not self.is_closed:
            await self.close()

    @property
    def is_closed(self) -> bool:
        """Indicates if the session is closed."""
        return self._closed

    async def close(self):
        """Closes the connection to chzzk."""
        self._closed = True
        await self._api_session.close()
        await self._game_session.close()

        for manage_client in self._manage_client.values():
            if manage_client.is_closed:
                continue

            await manage_client.close()
        return

    def login(self, authorization_key: str, session_key: str):
        """Login at Chzzk.
        Used for features that require a login. (ex. user method)

        Parameters
        ----------
        authorization_key : str
            A `NID_AUT` value in the cookie.
        session_key : str
            A `NID_SES` value in the cookie.
        """
        self._api_session.login(authorization_key, session_key)
        self._game_session.login(authorization_key, session_key)

    async def live_status(self, channel_id: str) -> Optional[LiveStatus]:
        """Get a live status info of broadcaster.

        Parameters
        ----------
        channel_id : str
            The channel ID of broadcaster

        Returns
        -------
        Optional[LiveStatus]
            Return LiveStatus info. Sometimes the broadcaster is not broadcasting, returns None.
        """
        res = await self._api_session.live_status(channel_id=channel_id)
        return res.content

    async def live_detail(self, channel_id: str) -> Optional[LiveDetail]:
        """Get a live detail info of broadcaster.

        Parameters
        ----------
        channel_id : str
            The channel ID of broadcaster

        Returns
        -------
        Optional[LiveDetail]
            Return LiveDetail info. Sometimes the broadcaster is not broadcasting, returns None.
        """
        res = await self._api_session.live_detail(channel_id=channel_id)
        return res.content

    async def user(self) -> User:
        """Get my user info.
        This method should be used after login.

        Returns
        -------
        User
            Information for logged-in user.
        """
        res = await self._game_session.user()
        return res.content

    async def search_channel(self, keyword: str) -> list[Channel]:
        """Search the channel with keyword.

        Parameters
        ----------
        keyword : str
            A keyword to search channel

        Returns
        -------
        list[Channel]
            Returns channels with searching.
        """
        res = await self._api_session.search_channel(keyword=keyword)
        data = res.content.data
        return [x.channel for x in data]

    async def search_video(self, keyword: str) -> list[Video]:
        """Search the video with keyword.

        Parameters
        ----------
        keyword : str
            A keyword to search video

        Returns
        -------
        list[Video]
            Returns videos with searching.
        """
        res = await self._api_session.search_video(keyword=keyword)
        data = res.content.data

        # Inject Channel info
        for i, x in enumerate(data):
            data[i].video.channel = x.channel

        return [x.video for x in data]

    async def search_live(self, keyword: str) -> list[Live]:
        """Search the live with keyword.

        Parameters
        ----------
        keyword : str
            A keyword to search live

        Returns
        -------
        list[Video]
            Returns lives with searching.
        """
        res = await self._api_session.search_live(keyword=keyword)
        data = res.content.data

        # Inject Channel info
        for i, x in enumerate(data):
            data[i].live.channel = x.channel

        return [x.live for x in data]

    async def autocomplete(self, keyword: str) -> list[str]:
        """Get a auto-completed keyword.

        Parameters
        ----------
        keyword : str
            Incomplete keywords

        Returns
        -------
        list[str]
            Autocompleted keywords
        """
        res = await self._api_session.autocomplete(keyword=keyword)
        data = res.content.data
        return data

    def manage(self, channel_id: Optional[str] = None) -> ManageClient:
        """Get a client provided broadcast management functionality.

        Parameters
        ----------
        channel_id : Optional[str]
            A channel id to manage broadcasts.
            The default value is the last channel id used.
            If initally use the manage method and don't have a channel_id argument,
            it will raise a :exc:`TypeError` exception.

        Returns
        -------
        ManageClient
            Return a client provided broadcast management functionality.
        """
        if channel_id is None:
            if self._latest_manage_client_id is None:
                raise TypeError("manage() missing 1 required argument: 'channel_id'")

            channel_id = self._latest_manage_client_id

        if channel_id not in self._manage_client.keys():
            self._manage_client[channel_id] = ManageClient(channel_id, self)
        self._latest_manage_client_id = channel_id
        return self._manage_client[channel_id]



"""MIT License

Copyright (c) 2024 gunyu1019

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import datetime
import functools
from typing import Optional, Literal, TypeVar, Generic, TYPE_CHECKING, Any
from pydantic import AliasChoices, Field, Json, ConfigDict, PrivateAttr

from .donation import (
    BaseDonation,
    ChatDonation,
    VideoDonation,
    MissionDonation,
    MissionParticipationDonation,
)
from .enums import ChatType
from .profile import Profile
from ..base_model import ChzzkModel

if TYPE_CHECKING:
    from .chat_client import ChatClient

E = TypeVar("E", bound="ExtraBase | BaseDonation")


class ExtraBase(ChzzkModel):
    pass


class Extra(ExtraBase):
    chat_type: str
    emojis: Optional[Any] = None
    os_type: Literal["PC", "AOS", "IOS"]
    streaming_channel_id: str


class Message(ChzzkModel, Generic[E]):
    service_id: str = Field(validation_alias=AliasChoices("serviceId", "svcid"))
    channel_id: str = Field(validation_alias=AliasChoices("channelId", "cid"))
    user_id: str = Field(validation_alias=AliasChoices("uid", "userId"))

    profile: Optional[Json[Profile]]
    content: str = Field(validation_alias=AliasChoices("msg", "content"))
    type: ChatType = Field(
        validation_alias=AliasChoices("msgTypeCode", "messageTypeCode")
    )
    extras: Optional[Json[E]]

    created_time: datetime.datetime = Field(
        validation_alias=AliasChoices("ctime", "createTime")
    )
    updated_time: Optional[datetime.datetime] = Field(
        default=None, validation_alias=AliasChoices("utime", "updateTime")
    )
    time: datetime.datetime = Field(
        validation_alias=AliasChoices("msgTime", "messageTime")
    )

    _client: Optional[ChatClient] = PrivateAttr(default=None)

    @staticmethod
    def _based_client(func):
        @functools.wraps(func)
        async def wrapper(self: MessageDetail, *args, **kwargs):
            if self._client is None:
                raise RuntimeError(
                    f"This {self.__class__.__name__} is intended to store message information only."
                )
            return await func(self, *args, **kwargs)

        return wrapper

    @classmethod
    def model_validate_with_client(
        cls: type[Message], obj: Any, client: ChatClient
    ) -> Message:
        model = super().model_validate(obj)
        model._client = client

        if model.profile is not None:
            model.profile._set_manage_client(client.manage_self)
        return model

    @property
    def is_me(self) -> bool:
        """Verify that this message is from a user signed in to the client."""
        if self._client is None:
            raise RuntimeError(
                f"This {self.__class__.__name__} is intended to store message information only."
            )
        return self._client.user_id == self.user_id

    @_based_client
    async def send(self, message: str):
        """Send message to broadcaster."""
        await self._client.send_chat(message)


class MessageDetail(Message[E], Generic[E]):
    member_count: int = Field(validation_alias=AliasChoices("mbrCnt", "memberCount"))
    message_status: Optional[str] = Field(
        validation_alias=AliasChoices("msgStatusType", "messageStatusType")
    )

    # message_tid: ???
    # session: bool

    @property
    def is_blind(self) -> bool:
        return self.message_status == "BLIND"


class ChatMessage(MessageDetail[Extra]):
    @Message._based_client
    async def pin(self):
        """Pin this message."""
        await self._client.set_notice_message(self)

    @Message._based_client
    async def unpin(self):
        """Unpin this message."""
        await self._client.delete_notice_message()

    @Message._based_client
    async def blind(self):
        """Blind this message."""
        await self._client.blind_message(self)

    @Message._based_client
    async def temporary_restrict(self):
        """Temporary restrict this user."""
        await self._client.temporary_restrict(self.profile)


class NoticeExtra(Extra):
    register_profile: Profile


class NoticeMessage(Message[NoticeExtra]):
    @Message._based_client
    async def unpin(self):
        """Unpin this message."""
        await self._client.delete_notice_message()


class ChatDonationExtra(ChatDonation):
    pass


class VideoDonationExtra(VideoDonation):
    pass


class MissionDonationExtra(MissionDonation):
    pass


class DonationMessage(
    MessageDetail[
        ChatDonationExtra
        | VideoDonationExtra
        | MissionDonationExtra
        | MissionParticipationDonation
    ]
):
    pass


class SubscriptionExtra(ExtraBase):
    month: int
    tier_name: str
    nickname: Optional[str] = None
    tier_no: Optional[int] = None


class SubscriptionMessage(MessageDetail[SubscriptionExtra]):
    pass


class SystemExtraParameter(ChzzkModel):
    register_nickname: str
    target_nickname: str
    register_chat_profile: Optional[Json[Profile]] = Field(
        alias="registerChatProfileJson", default=None
    )
    target_profile: Optional[Json[Profile]] = Field(
        alias="targetChatProfileJson", default=None
    )


class SystemExtra(ExtraBase):
    description: str
    style_type: int
    visible_roles: list[str]
    params: Optional[SystemExtraParameter] = None


class SystemMessage(MessageDetail[SystemExtra]):
    pass



"""MIT License

Copyright (c) 2024 gunyu1019

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import aiohttp
from typing import Optional
from ..error import ChzzkpyException


class ChatConnectFailed(ChzzkpyException):
    def __init__(self, message: str):
        super(ChatConnectFailed, self).__init__(message)

    @classmethod
    def channel_is_null(cls, channel_id: str):
        return cls(
            f"Client can't retrieve the channel live-status. "
            f"Is the channel({channel_id}) broadcasting live?"
        )

    @classmethod
    def chat_channel_is_null(cls):
        return cls(
            "Missing Chat ID to connect to chat."
            "Make sure this client can connect to chat."
        )

    @classmethod
    def adult_channel(cls, channel_id: str):
        return cls(
            f"Adult verification is required to connect to chat on this channel({channel_id})."
            f"Please use login(), or proceed to adult verification."
        )


class ConnectionClosed(Exception):
    def __init__(
        self, socket: aiohttp.ClientWebSocketResponse, code: Optional[int] = None
    ):
        self.code: int = code or socket.close_code or -1
        self.reason: str = ""
        super().__init__(f"WebSocket closed with {self.code}")


class WebSocketClosure(Exception):
    pass


class ReconnectWebsocket(Exception):
    pass



"""MIT License

Copyright (c) 2024 gunyu1019

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import datetime

from typing import Optional, Literal
from pydantic import AliasChoices, Field

from ..base_model import ChzzkModel


class DonationRank(ChzzkModel):
    user_id_hash: str
    nickname: str = Field(validation_alias=AliasChoices("nickname", "nickName"))
    verified_mark: bool
    donation_amount: int
    ranking: int


class BaseDonation(ChzzkModel):
    is_anonymous: bool = True
    pay_type: str
    pay_amount: int = 0
    donation_type: str
    weekly_rank_list: Optional[list[DonationRank]] = Field(default_factory=list)
    donation_user_weekly_rank: Optional[DonationRank] = None
    verified_mark: Optional[bool] = False


class ChatDonation(BaseDonation):
    donation_type: Literal["CHAT"]


class VideoDonation(BaseDonation):
    donation_type: Literal["VIDEO"]


class MissionDonation(BaseDonation):
    donation_type: Literal["MISSION"]
    mission_donation_id: Optional[str] = None
    mission_donation_type: Optional[str] = None  # ALONE / GROUP
    mission_text: str
    total_pay_amount: int

    donation_id: Optional[str]  # ???
    participation_count: int

    user_id_hash: Optional[str] = None
    nickname: Optional[str] = None
    anonymous_token: Optional[str] = None

    mission_created_time: datetime.datetime
    mission_start_time: Optional[datetime.datetime] = None
    mission_end_time: Optional[datetime.datetime] = None
    duration_time: int = 0

    status: str | Literal["PENDING", "REJECTED", "APPROVED", "COMPLETED"] = None
    success: bool = False


class MissionParticipationDonation(BaseDonation):
    donation_type: Literal["MISSION_PARTICIPATION"]
    mission_donation_id: Optional[str] = None
    mission_donation_type: Optional[str] = None  # PARTICIPATION
    mission_text: str
    total_pay_amount: int

    related_mission_donation_id: str
    donation_id: Optional[str]  # ???
    participation_count: int

    user_id_hash: Optional[str] = None
    nickname: Optional[str] = None
    anonymous_token: Optional[str] = None

    status: str | Literal["PENDING", "REJECTED", "APPROVED", "COMPLETED"] = None
    success: bool = False



"""MIT License

Copyright (c) 2024 gunyu1019

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

from typing import List, Literal, Optional, TYPE_CHECKING
from ..error import LoginRequired
from ..user import PartialUser, UserRole
from .enums import SortType, SubscriberTier
from .http import ChzzkManageSession
from .chat_activity_count import ChatActivityCount
from .prohibit_word import ProhibitWord
from .stream import Stream
from .manage_search import (
    ManageResult,
    ManageSubcriber,
    ManageFollower,
    RestrictUser,
    UnrestrictRequest,
    ManageVideo,
)

if TYPE_CHECKING:
    from ..client import Client


class ManageClient:
    """Represent a client that provides broadcast management functionality."""

    def __init__(self, channel_id: str, client: Client):
        self.channel_id = channel_id
        self.client = client

        # All manage feature needs login.
        if not self.client._api_session.has_login:
            raise LoginRequired()

        self._http = ChzzkManageSession(self.client.loop)
        self._http.login(
            authorization_key=self.client._api_session._authorization_key,
            session_key=self.client._api_session._session_key,
        )
        self._is_closed = False

    async def close(self):
        """Closes the connection to chzzk."""
        self._is_closed = True
        await self._http.close()
        return

    @property
    def is_closed(self) -> bool:
        """Indicates if the session is closed."""
        return self._is_closed

    async def get_prohibit_words(self) -> List[ProhibitWord]:
        """Get prohibit words in chat.

        Returns
        -------
        List[ProhibitWord]
            Returns the prohibit words.
        """
        data = await self._http.get_prohibit_words(self.channel_id)
        prohibit_words = [
            x.set_manage_client(self) for x in data.content.prohibit_word_list
        ]
        return prohibit_words

    async def get_prohbit_word(self, word: str) -> Optional[ProhibitWord]:
        """Get prohibit word with word.
        When word does not contain prohibit word, returns None.

        Parameters
        ----------
        word : str
            A word to find prohibit word.

        Returns
        -------
        Optional[ProhibitWord]
            When word contains prohibit words, return :class:`ProhibitWord` object.
        """
        data = await self.get_prohibit_words()
        prohibit_words = [x for x in data if x.prohibit_word == word]
        if len(prohibit_words) <= 0:
            return
        return prohibit_words[0]

    async def add_prohibit_word(self, word: str) -> Optional[ProhibitWord]:
        """Add a prohibit word at chat.

        Parameters
        ----------
        word : str
            A word to prohibit.

        Returns
        -------
        Optional[ProhibitWord]
            Returns the generated prohibit word.
        """
        await self._http.add_prohibit_word(self.channel_id, word)
        return await self.get_prohbit_word(word)

    async def edit_prohibit_word(
        self, prohibit_word: ProhibitWord | int, word: str
    ) -> Optional[ProhibitWord]:
        """Modify a prohibit word.

        Parameters
        ----------
        prohibit_word : ProhibitWord | int
            The prohibit word object to modify.
            Instead, it can be prohibit word id.
        word : str
            A new word to prohibit.

        Returns
        -------
        Optional[ProhibitWord]
            Returns the modified prohibit word.
        """
        if isinstance(prohibit_word, ProhibitWord):
            prohibit_word_number = prohibit_word.prohibit_word_no
        else:
            prohibit_word_number = prohibit_word

        await self._http.edit_prohibit_word(self.channel_id, prohibit_word_number, word)
        return await self.get_prohbit_word(word)

    async def remove_prohibit_word(self, prohibit_word: ProhibitWord | int) -> None:
        """Remove a prohibit word.

        Parameters
        ----------
        prohibit_word : ProhibitWord | int
            The prohibit word object to remove.
            Instead, it can be prohibit word id.
        """
        if isinstance(prohibit_word, ProhibitWord):
            prohibit_word_number = prohibit_word.prohibit_word_no
        else:
            prohibit_word_number = prohibit_word

        await self._http.remove_prohibit_word(self.channel_id, prohibit_word_number)

    async def remove_prohibit_words(self) -> None:
        """Remove all prohibit words."""
        await self._http.remove_prohibit_word_all(self.channel_id)

    async def get_chat_rule(self) -> str:
        """Get chat rule of broadcast.

        Returns
        -------
        str
            Returns a chat rule.
        """
        data = await self._http.get_chat_rule(self.channel_id)
        return data.content.rule

    async def set_chat_rule(self, word: str) -> None:
        """Set chat rule of broadcast.

        Parameters
        ----------
        word : str
            A new chat rule to set up.
        """
        await self._http.set_chat_rule(self.channel_id, word)

    async def stream(self) -> Stream:
        """Get a stream key required for streamming.

        Returns
        -------
        Stream
            Return a stream key for streamming.
        """
        data = await self._http.stream(channel_id=self.channel_id)
        return data.content

    async def add_restrict(self, user: str | PartialUser) -> PartialUser:
        """Add an user to restrict activity.

        Parameters
        ----------
        user : str | ParticleUser
            A user object to add restrict activity.
            Instead, it can be user id or nickname.

        Returns
        -------
        ParticleUser
            Returns an object containning activity-restricted users.
        """
        target_id = user
        if isinstance(user, PartialUser):
            target_id = user.user_id_hash

        data = await self._http.add_restrict(
            channel_id=self.channel_id, target_id=target_id
        )
        user = data.content
        user._set_manage_client(self)
        return user

    async def remove_restrict(self, user: str | PartialUser) -> None:
        """Remove an user to restrict activity.

        Parameters
        ----------
        user : str | ParticleUser
            A user object to remove restrict activity.
            Instead, it can be user id or nickname.

        Returns
        -------
        ParticleUser
            Returns an user whose activity is unrestricted.
        """
        target_id = user
        if isinstance(user, PartialUser):
            target_id = user.user_id_hash

        await self._http.remove_restrict(
            channel_id=self.channel_id, target_id=target_id
        )

    async def add_role(self, user: str | PartialUser, role: UserRole) -> PartialUser:
        """Add a broadcast permission to user.

        Parameters
        ----------
        user : str | ParticleUser
            A user object to add role.
            Instead, it can be user id or nickname.
        role : UserRole
            A enumeration class containing broadcast role.
            It can only set the role to :attr:`UserRole.chat_manager`,
            :attr:`UserRole.settlement_manager`, or :attr:`UserRole.channel_manager`.
            Giving any other role will cause a :exc:`TypeError` exception.

        Returns
        -------
        ParticleUser
            Returns an user with added role.
        """
        user_id = user
        if isinstance(user, PartialUser):
            user_id = user.user_id_hash

        if role in [UserRole.common_user, UserRole.streamer, UserRole.manager]:
            raise TypeError(f"You cannot give role({role.name}) to user.")

        data = await self._http.add_role(
            channel_id=self.channel_id,
            target_id=user_id,
            user_role_type=role.value.upper(),
        )
        user = data.content
        user._set_manage_client(self)
        return user

    async def remove_role(self, user: str | PartialUser) -> None:
        """Remove a broadcast permission to user.

        Parameters
        ----------
        user : str | ParticleUser
            A user object to remove role.
            Instead, it can be user id or nickname.
        """
        user_id = user
        if isinstance(user, PartialUser):
            user_id = user.user_id_hash

        await self._http.remove_role(channel_id=self.channel_id, target_id=user_id)

    async def chat_activity_count(self, user: str | PartialUser) -> ChatActivityCount:
        """Get chat activity count of user.

        Parameters
        ----------
        user : str | ParticleUser
            A user object to get chat activity count.
            Instead, it can be user id.

        Returns
        -------
        ChatActivityCount
            Returns a chat activity count object contains the count of temporary activity restrictions,
        the count of activity restrictions, and the count of chats.
        """
        user_id = user
        if isinstance(user, PartialUser):
            user_id = user.user_id_hash

        data = await self._http.chat_activity_count(
            channel_id=self.channel_id, target_id=user_id
        )
        return data.content

    async def subcribers(
        self,
        page: int = 0,
        size: int = 50,
        sort_type: SortType = SortType.recent,
        publish_period: Optional[Literal[1, 3, 6]] = None,
        tier: Optional[SubscriberTier] = None,
        nickname: Optional[str] = None,
    ) -> ManageResult[ManageSubcriber]:
        """Get subcribers of channel.

        Parameters
        ----------
        page : Optional[int]
            The number of page, by default 0
        size : Optional[int]
            The number of subscribers to import at once, by default 50
        sort_type : Optional[SortType]
            A sort order, by default SortType.recent
        publish_period : Optional[Literal[1, 3, 6]]
            Lookup by the subscriber's publish period, by default None
        tier : Optional[SubscriberTier]
            Lookup by the subscriber's tier, by default None.
        nickname : Optional[str]
            Lookup by the subscriber's nickname, by default None

        Returns
        -------
        ManageResult[ManageSubcriber]
            Returns a :class:`ManageResult` containing the subscriber info.
        """
        data = await self._http.subcribers(
            channel_id=self.channel_id,
            page=page,
            size=size,
            sort_type=sort_type.value,
            publish_period="" if publish_period is None else publish_period,
            tier="" if tier is None else tier.value,
            user_nickname="" if nickname is None else nickname,
        )
        return data.content

    async def followers(
        self, page: int = 0, size: int = 50, sort_type: SortType = SortType.recent
    ) -> ManageResult[ManageFollower]:
        """Get followers of channel.

        Parameters
        ----------
        page : Optional[int]
            The number of page, by default 0
        size : Optional[int]
            The number of followers to import at once, by default 50
        sort_type : Optional[SortType]
            A sort order, by default SortType.recent

        Returns
        -------
        ManageResult[ManageFollower]
            Returns a :class:`ManageResult` containing the follower info.
        """
        data = await self._http.followers(
            channel_id=self.channel_id, page=page, size=size, sort_type=sort_type.value
        )
        for followed_user in data.content.data:
            followed_user.user._set_manage_client(self)
        return data.content

    async def restrict(
        self, page: int = 0, size: int = 50, nickname: Optional[str] = None
    ) -> ManageResult[UnrestrictRequest]:
        """Get activitiy restricted user of channel.

        Parameters
        ----------
        page : Optional[int]
            The number of page, by default 0
        size : Optional[int]
            The number of activity restricted user to import at once, by default 50
        nickname : Optional[str]
            Lookup by the activity restricted user's nickname, by default None

        Returns
        -------
        ManageResult[RestrictUser]
            Returns a :class:`ManageResult` containing the restricted user info.
        """
        data = await self._http.restricts(
            channel_id=self.channel_id,
            page=page,
            size=size,
            user_nickname="" if nickname is None else nickname,
        )
        for restricted_user in data.content.data:
            restricted_user._set_manage_client(self)
        return data.content

    async def unrestrict_requests(
        self, page: int = 0, size: int = 50, nickname: Optional[str] = None
    ) -> ManageResult[UnrestrictRequest]:
        """Get unrestrict activity requests of channel.

        Parameters
        ----------
        page : Optional[int]
            The number of page, by default 0
        size : Optional[int]
            The number of unrestrict activity requests to import at once, by default 50
        nickname : Optional[str]
            Lookup by the unrestrict activity requests with user's nickname, by default None

        Returns
        -------
        ManageResult[UnrestrictRequest]
            Returns a :class:`ManageResult` containing the unrestrict requests.
        """
        data = await self._http.unrestrict_requests(
            channel_id=self.channel_id,
            page=page,
            size=size,
            user_nickname="" if nickname is None else nickname,
        )
        for unrestrict_request in data.content.data:
            unrestrict_request._set_manage_client(self)
        return data.content

    async def live_replay(
        self, page: int = 0, size: int = 50
    ) -> ManageResult[ManageVideo]:
        """Get streamming replay video of channel.

        Parameters
        ----------
        page : Optional[int]
            The number of page, by default 0
        size : Optional[int]
            The number of streamming replay video to import at once, by default 50

        Returns
        -------
        ManageResult[ManageVideo]
            Returns a :class:`ManageResult` containing the streamming replay video.
        """
        data = await self._http.videos(
            channel_id=self.channel_id, video_type="REPLAY", page=page, size=size
        )
        return data.content

    async def videos(self, page: int = 0, size: int = 50) -> ManageResult[ManageVideo]:
        """Get uploaded video of channel.

        Parameters
        ----------
        page : Optional[int]
            The number of page, by default 0
        size : Optional[int]
            The number of video to import at once, by default 50

        Returns
        -------
        ManageResult[ManageVideo]
            Returns a :class:`ManageResult` containing the video.
        """
        data = await self._http.videos(
            channel_id=self.channel_id, video_type="UPLOAD", page=page, size=size
        )
        return data.content



"""MIT License

Copyright (c) 2024 gunyu1019

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import datetime
from enum import Enum
from typing import Annotated, Any, Optional, TYPE_CHECKING

from pydantic import BeforeValidator

from .base_model import ChzzkModel, ManagerClientAccessable

if TYPE_CHECKING:
    from .manage.chat_activity_count import ChatActivityCount


class UserRole(Enum):
    common_user = "common_user"
    streamer = "streamer"
    chat_manager = "streaming_chat_manager"
    channel_manager = "streaming_channel_manager"
    settlement_manager = "streaming_settlement_manager"
    manager = "manager"


class PartialUser(ChzzkModel, ManagerClientAccessable):
    user_id_hash: Optional[str]
    nickname: Optional[str]
    profile_image_url: Optional[str]
    verified_mark: bool = False

    @ManagerClientAccessable.based_manage_client
    async def add_restrict(self):
        """Add this user to restrict activity."""
        result = await self._manage_client.add_restrict(self)
        return result

    @ManagerClientAccessable.based_manage_client
    async def remove_restrict(self):
        """Remove this user to restrict activity."""
        await self._manage_client.remove_restrict(self)

    @ManagerClientAccessable.based_manage_client
    async def add_role(self, role: UserRole):
        """Add a broadcast permission to this user.

        Parameters
        ----------
        role : UserRole
            A enumeration class containing broadcast role.
            It can only set the role to :attr:`UserRole.chat_manager`,
            :attr:`UserRole.settlement_manager`, or :attr:`UserRole.channel_manager`.
            Giving any other role will cause a :exc:`TypeError` exception.
        """
        result = await self._manage_client.add_role(self, role)
        return result

    @ManagerClientAccessable.based_manage_client
    async def remove_role(self):
        """Remove a broadcast permission to this user."""
        await self._manage_client.remove_role(self)

    @ManagerClientAccessable.based_manage_client
    async def chat_activity_count(self) -> ChatActivityCount:
        """Get chat activity count of this user.

        Returns
        -------
        ChatActivityCount
            Returns a chat activity count object contains the count of temporary activity restrictions,
            the count of activity restrictions, and the count of chats.
        """
        data = await self._manage_client.chat_activity_count(self)
        return data


class User(PartialUser):
    has_profile: bool
    penalties: Optional[list[Any]]  # typing: ???
    official_noti_agree: bool
    official_noti_agree_updated_date: Annotated[
        Optional[datetime.datetime],
        BeforeValidator(ChzzkModel.special_date_parsing_validator),
    ]  # Example: YYYY-MM-DDTHH:MM:SS.SSS+09
    logged_in: Optional[bool]



"""MIT License

Copyright (c) 2024 gunyu1019

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import logging
import inspect
import functools
from typing import Callable, Any, TYPE_CHECKING, Optional

from .blind import Blind
from .donation import MissionDonation, MissionParticipationDonation
from .enums import ChatCmd, ChatType, get_enum
from .message import (
    ChatMessage,
    DonationMessage,
    NoticeMessage,
    SubscriptionMessage,
    SystemMessage,
)
from .recent_chat import RecentChat

if TYPE_CHECKING:
    from .chat_client import ChatClient

log = logging.getLogger()


class ConnectionState:
    def __init__(
        self,
        dispatch: Callable[..., Any],
        handler: dict[ChatCmd | int, Callable[..., Any]],
        client: Optional[ChatClient] = None,
    ):
        self.dispatch = dispatch
        self.handler: dict[ChatCmd | int, Callable[..., Any]] = handler
        self.parsers: dict[ChatCmd, Callable[..., Any]] = dict()
        for _, func in inspect.getmembers(self):
            if hasattr(func, "__parsing_event__"):
                self.parsers[func.__parsing_event__] = func

        self.client = client

    @staticmethod
    def parsable(cmd: ChatCmd):
        def decorator(func: Callable[..., Any]):
            func.__parsing_event__ = cmd
            return func

        return decorator

    @staticmethod
    def catch_exception(func: Callable[..., Any]):
        @functools.wraps(func)
        def wrapper(self, *args, **kwargs):
            try:
                return func(self, *args, **kwargs)
            except Exception as exc:
                self.dispatch("client_error", exc, *args, **kwargs)
                log.exception(exc)

        return wrapper

    def call_handler(self, key: ChatCmd, *args: Any, **kwargs: Any):
        if key in self.handler:
            func = self.handler[key]
            func(*args, **kwargs)

    @parsable(ChatCmd.CONNECTED)
    @catch_exception
    def parse_connect(self, _: dict[str, Any]):
        self.call_handler(ChatCmd.CONNECTED)
        self.dispatch("connect")

    def _parse_all_type_of_chat(self, data: list[dict[str, Any]]):
        if data is None or len(data) == 0:
            return

        for message in data:
            message_raw_type = message.get("messageTypeCode") or message.get(
                "msgTypeCode"
            )
            message_type = get_enum(ChatType, message_raw_type)

            # Cause bug from insufficient information
            # ChatType: SYSTEM_MESSAGE
            if message.get("profile") == "{}":
                message["profile"] = None

            if message_type == ChatType.DONATION:
                validated_data = DonationMessage.model_validate_with_client(
                    message, client=self.client
                )
                self.dispatch("donation", validated_data)
            elif message_type == ChatType.SYSTEM_MESSAGE:

                validated_data = SystemMessage.model_validate_with_client(
                    message, client=self.client
                )
                self.dispatch("system_message", validated_data)
            elif message_type == ChatType.TEXT:
                validated_data = ChatMessage.model_validate_with_client(
                    message, client=self.client
                )
                self.dispatch("chat", validated_data)
            elif message_type == ChatType.SUBSCRIPTION:
                validated_data = SubscriptionMessage.model_validate_with_client(
                    message, client=self.client
                )
                self.dispatch("subscription", validated_data)

    @parsable(ChatCmd.CHAT)
    @catch_exception
    def parse_chat(self, data: list[dict[str, Any]]):
        self._parse_all_type_of_chat(data)

    @parsable(ChatCmd.RECENT_CHAT)
    @catch_exception
    def parse_recent_chat(self, data: dict[str, Any]):
        validated_data = RecentChat.model_validate(data)
        self.dispatch("recent_chat", validated_data)

    @parsable(ChatCmd.SPECIAL_CHAT)
    @catch_exception
    def parse_special_chat(self, data: list[dict[str, Any]]):
        self._parse_all_type_of_chat(data)

    @parsable(ChatCmd.NOTICE)
    @catch_exception
    def parse_notice(self, data: dict[str, Any]):
        if len(data) == 0:
            self.dispatch("unpin")
            return
        validated_data = NoticeMessage.model_validate_with_client(
            data, client=self.client
        )
        self.dispatch("notice", validated_data)
        self.dispatch("pin", validated_data)

    @parsable(ChatCmd.BLIND)
    @catch_exception
    def parse_blind(self, data: dict[str, Any]):
        validated_data = Blind.model_validate(data)
        self.dispatch("blind", validated_data)

    @parsable(ChatCmd.EVENT)
    @catch_exception
    def parse_event(self, data: dict[str, Any]):  # For mission donation handler
        event_type = data.get("type")

        if event_type == "DONATION_MISSION_IN_PROGRESS":
            validated_data = MissionDonation.model_validate(data)
            if validated_data.status == "COMPLETED":
                self.dispatch("mission_completed", validated_data)
            elif validated_data.status == "PENDING":
                self.dispatch("mission_pending", validated_data)
            elif validated_data.status == "APPROVED":
                self.dispatch("mission_approved", validated_data)
            elif validated_data.status == "REJECTED":
                self.dispatch("mission_rejected", validated_data)
        elif (
            event_type == "DONATION_MISSION_PARTICIPATION"
            and "relatedMissionDonationId" in data.keys()
        ):
            validated_data = MissionParticipationDonation.model_validate(data)
            self.dispatch("mission_update_cost", validated_data)
        return



import asyncio

from ahttp_client import get, post, delete, Query, Path
from ahttp_client.extension import get_pydantic_response_model
from typing import Annotated, Optional

from .access_token import AccessToken
from .profile import Profile
from ..base_model import Content
from ..http import ChzzkSession, ChzzkAPISession, NaverGameAPISession
from ..user import PartialUser


class ChzzkAPIChatSession(ChzzkAPISession):
    def __init__(self, loop: Optional[asyncio.AbstractEventLoop] = None):
        super().__init__(loop=loop)

        self.temporary_restrict.before_hook(self.query_to_json)

    @get_pydantic_response_model()
    @post(
        "/manage/v1/channels/{channel_id}/temporary-restrict-users",
        directly_response=True,
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    async def temporary_restrict(
        self,
        channel_id: Annotated[str, Path],
        chat_channel_id: Annotated[str, Query.to_camel()],
        target_id: Annotated[str, Query.to_camel()],
    ) -> Content[PartialUser]:
        pass


class NaverGameChatSession(NaverGameAPISession):
    def __init__(self, loop: Optional[asyncio.AbstractEventLoop] = None):
        super().__init__(loop=loop)

        self.delete_notice_message.before_hook(ChzzkSession.query_to_json)
        self.set_notice_message.before_hook(ChzzkSession.query_to_json)
        self.blind_message.before_hook(ChzzkSession.query_to_json)

    @get_pydantic_response_model()
    @get("/nng_main/v1/chats/access-token", directly_response=True)
    @ChzzkSession.configuration(login_able=True)
    @Query.default_query("chatType", "STREAMING")
    async def chat_access_token(
        self, channel_id: Annotated[str, Query.to_camel()]
    ) -> Content[AccessToken]:
        pass

    @get_pydantic_response_model()
    @delete("/nng_main/v1/chats/notices", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    @Query.default_query("chatType", "STREAMING")
    async def delete_notice_message(
        self, channel_id: Annotated[str, Query.to_camel()]
    ) -> Content[None]:
        pass

    @get_pydantic_response_model()
    @post("/nng_main/v1/chats/notices", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    @Query.default_query("chatType", "STREAMING")
    async def set_notice_message(
        self,
        channel_id: Annotated[str, Query.to_camel()],
        extras: Annotated[str, Query],
        message: Annotated[str, Query],
        message_time: Annotated[int, Query.to_camel()],
        message_user_id_hash: Annotated[int, Query.to_camel()],
        streaming_channel_id: Annotated[int, Query.to_camel()],
    ) -> Content[None]:
        return

    @get_pydantic_response_model()
    @post("/nng_main/v1/chats/blind-message", directly_response=True)
    @ChzzkSession.configuration(login_able=True, login_required=True)
    @Query.default_query("chatType", "STREAMING")
    async def blind_message(
        self,
        channel_id: Annotated[str, Query.to_camel()],
        message: Annotated[str, Query],
        message_time: Annotated[int, Query.to_camel()],
        message_user_id_hash: Annotated[int, Query.to_camel()],
        streaming_channel_id: Annotated[int, Query.to_camel()],
    ) -> Content[None]:
        pass

    @get_pydantic_response_model()
    @get(
        "/nng_main/v1/chats/{chat_channel_id}/users/{user_id}/profile-card",
        directly_response=True,
    )
    @ChzzkSession.configuration(login_able=True, login_required=True)
    @Query.default_query("chatType", "STREAMING")
    async def profile_card(
        self,
        chat_channel_id: Annotated[str, Path],
        user_id: Annotated[str, Path],
    ) -> Content[Profile]:
        pass



import re

from setuptools import setup

version = ""
with open("chzzkpy/__init__.py") as f:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE
    ).group(1)

if not version:
    raise RuntimeError("version is not set")


extras_require = {"test": ["pytest", "pytest-cov"], "lint": ["pycodestyle", "black"], "docs": ["Sphinx", "sphinxawesome-theme", "sphinx-intl"]}

setup(
    name="chzzkpy",
    version=version,
    packages=["chzzkpy", "chzzkpy.chat", "chzzkpy.manage"],
    url="https://github.com/gunyu1019/chzzk_py",
    license="MIT",
    author="gunyu1019",
    author_email="gunyu1019@gmail.com",
    description="An unofficial Python library of Chzzk(Naver live-streaming service)",
    python_requires=">=3.10",
    extras_require=extras_require,
    long_description=open("README.md", encoding="UTF-8").read(),
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=open("requirements.txt", encoding="UTF-8").read(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Natural Language :: Korean",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
    ],
)
